PROGRAM main

  !-----------------------------------------------------------------------------
  !   This program regrids POP dipole grid data to a regular lat-lon grid
  !   using SCRIP-computed remapping weights stored in 'map_file', 
  !   and optionally, to a standard vertical grid.  Seperate SCRIP remapping 
  !   files should exist for each k-level of the native POP grid.
  ! 
  !   Usage:   
  !		src_file  == POP output on original grid
  !	        dst_file  == Regridded output netcdf (1 variable per file)
  !		dst_lclobber == TRUE allows clobbering dst_file
  !		dst_z_standard == TRUE regrids to a standard z grid
  !				  FALSE preserves native z grid
  !		z_file    == data file which defines the standard z grid
  !		nk        == # of native z levels to process
  !		src_var   == variable to be processed
  !		map_file  == SCRIP horizontal remapping file template 
  !
  !   If dst_z_standard = .true. remaps vertically to a standard
  !   grid given by z_std, otherwise just performs horizontal
  !   regridding down to nk levels (must have nk corresponding
  !   mapping files).
  !
  !   (Based on K. Lindsay's /fs/cgd/data0/klindsay/POP_tools/regrid )
  !
  !   CVS:$Id: main.F90,v 1.1.1.1 2004/09/13 20:11:32 yeager Exp $
  !-----------------------------------------------------------------------------

  USE kinds_mod
  USE vars_mod
  USE popregrid

  !*****************************************************************************

  IMPLICIT NONE

  !*****************************************************************************

  NAMELIST /popregrid_nml/ &
       src_file, src_var, &
       nk, map_file, z_file, &
       dst_file, dst_lclobber, dst_z_standard, dst_var

  INTEGER(KIND=int_kind), PARAMETER :: stdin = 5

  INTEGER(KIND=int_kind) :: ios

  !*****************************************************************************

  src_file = 'unknown'
  src_var  = 'unknown'
  dst_file = 'unknown'
  dst_lclobber = .TRUE.
  dst_z_standard = .FALSE.
  map_file = 'unknown'
  z_file = 'unknown'
  nk = 1

  DO
     dst_var = 'unknown'
     READ(UNIT=stdin, NML=popregrid_nml, IOSTAT=ios, ERR=10, END=20)
     IF (dst_var == 'unknown') dst_var = src_var
     PRINT *, 'src_file = ', TRIM(src_file)
     PRINT *, 'dst_file = ', TRIM(dst_file)
     PRINT *, 'src_var  = ', TRIM(src_var)
     PRINT *, 'map_file  = ', TRIM(map_file)
     PRINT *, 'dst_z_standard  = ', dst_z_standard
     PRINT *, 'z_file  = ', TRIM(z_file)
     CALL regrid_driver
     PRINT *, ' '
     CYCLE
10   PRINT *, 'error reading namelist, iostat = ', ios
20   EXIT
  END DO

END PROGRAM main
